@@ -1,4 +1,4 @@
 from .dataset_retrieval import load_dataset, get_available_datasets, \
     get_dataset_citations, get_dataset_column_description, \
     get_dataset_reference, get_dataset_columns, get_dataset_num_entries, \
-    get_dataset_description, get_dataset_attribute
+    get_dataset_description, get_dataset_attribute, get_all_dataset_info
