@@ -12,11 +12,13 @@ from pymatgen.core.periodic_table import Specie, Element
 from pymatgen.util.testing import PymatgenTest
 
 from matminer.featurizers.composition import Stoichiometry, ElementProperty, \
-    ValenceOrbital, IonProperty, \
-    ElementFraction, TMetalFraction, ElectronAffinity, ElectronegativityDiff, \
-    CohesiveEnergy, \
-    BandCenter, Miedema, CationProperty, OxidationStates, AtomicOrbitals, \
-    YangSolidSolution, AtomicPackingEfficiency
+    ValenceOrbital, IonProperty, ElementFraction, TMetalFraction, \
+    ElectronAffinity, \
+    ElectronegativityDiff, CohesiveEnergy, BandCenter, Miedema, CationProperty, \
+    OxidationStates, \
+    AtomicOrbitals, YangSolidSolution, AtomicPackingEfficiency, is_ionic, \
+    Meredig, CohesiveEnergyMP
+from matminer.featurizers.conversions import CompositionToOxidComposition
 
 
 class CompositionFeaturesTest(PymatgenTest):
@@ -25,6 +27,12 @@ class CompositionFeaturesTest(PymatgenTest):
         self.df = pd.DataFrame({"composition": [Composition("Fe2O3"),
                                                 Composition({Specie("Fe", 2): 1, Specie("O", -2): 1})]})
 
+    def test_is_ionic(self):
+        """Test checking whether a compound is ionic"""
+
+        self.assertTrue(is_ionic(Composition({Specie("Fe", 2): 1, Specie("O", -2): 1})))
+        self.assertFalse(is_ionic(Composition({Specie("Fe", 0): 1, Specie("Al", 0): 1})))
+
     def test_stoich(self):
         featurizer = Stoichiometry(num_atoms=True)
         df_stoich = Stoichiometry(num_atoms=True).featurize_dataframe(self.df, col_id="composition")
@@ -39,37 +47,66 @@ class CompositionFeaturesTest(PymatgenTest):
 
     def test_elem(self):
         df_elem = ElementProperty.from_preset("magpie").featurize_dataframe(self.df, col_id="composition")
-        self.assertAlmostEqual(df_elem["minimum Number"][0], 8)
-        self.assertAlmostEqual(df_elem["maximum Number"][0], 26)
-        self.assertAlmostEqual(df_elem["range Number"][0], 18)
-        self.assertAlmostEqual(df_elem["mean Number"][0], 15.2)
-        self.assertAlmostEqual(df_elem["avg_dev Number"][0], 8.64)
-        self.assertAlmostEqual(df_elem["mode Number"][0], 8)
+        self.assertAlmostEqual(df_elem["MagpieData minimum Number"][0], 8)
+        self.assertAlmostEqual(df_elem["MagpieData maximum Number"][0], 26)
+        self.assertAlmostEqual(df_elem["MagpieData range Number"][0], 18)
+        self.assertAlmostEqual(df_elem["MagpieData mean Number"][0], 15.2)
+        self.assertAlmostEqual(df_elem["MagpieData avg_dev Number"][0], 8.64)
+        self.assertAlmostEqual(df_elem["MagpieData mode Number"][0], 8)
 
     def test_elem_deml(self):
         df_elem_deml = ElementProperty.from_preset("deml").featurize_dataframe(self.df, col_id="composition")
-        self.assertAlmostEqual(df_elem_deml["minimum atom_num"][0], 8)
-        self.assertAlmostEqual(df_elem_deml["maximum atom_num"][0], 26)
-        self.assertAlmostEqual(df_elem_deml["range atom_num"][0], 18)
-        self.assertAlmostEqual(df_elem_deml["mean atom_num"][0], 15.2)
-        self.assertAlmostEqual(df_elem_deml["std_dev atom_num"][0], 12.7279, 4)
+        self.assertAlmostEqual(df_elem_deml["DemlData minimum atom_num"][0], 8)
+        self.assertAlmostEqual(df_elem_deml["DemlData maximum atom_num"][0], 26)
+        self.assertAlmostEqual(df_elem_deml["DemlData range atom_num"][0], 18)
+        self.assertAlmostEqual(df_elem_deml["DemlData mean atom_num"][0], 15.2)
+        self.assertAlmostEqual(df_elem_deml["DemlData std_dev atom_num"][0], 12.7279, 4)
 
     def test_cation_properties(self):
         featurizer = CationProperty.from_preset("deml")
         features = dict(zip(featurizer.feature_labels(), featurizer.featurize(self.df["composition"][1])))
-        self.assertAlmostEqual(features["minimum magn_moment of cations"], 5.48)
-        self.assertAlmostEqual(features["maximum magn_moment of cations"], 5.48)
-        self.assertAlmostEqual(features["range magn_moment of cations"], 0)
-        self.assertAlmostEqual(features["mean magn_moment of cations"], 5.48)
-        self.assertAlmostEqual(features["std_dev magn_moment of cations"], 0)
+        self.assertAlmostEqual(features["DemlData minimum magn_moment of cations"], 5.48)
+        self.assertAlmostEqual(features["DemlData maximum magn_moment of cations"], 5.48)
+        self.assertAlmostEqual(features["DemlData range magn_moment of cations"], 0)
+        self.assertAlmostEqual(features["DemlData mean magn_moment of cations"], 5.48)
+        self.assertAlmostEqual(features["DemlData std_dev magn_moment of cations"], 0)
 
     def test_elem_matminer(self):
         df_elem = ElementProperty.from_preset("matminer").featurize_dataframe(self.df, col_id="composition")
-        self.assertAlmostEqual(df_elem["minimum melting_point"][0], 54.8, 1)
-        self.assertTrue(math.isnan(df_elem["maximum bulk_modulus"][0]))
-        self.assertAlmostEqual(df_elem["range X"][0], 1.61, 1)
-        self.assertAlmostEqual(df_elem["mean X"][0], 2.796, 1)
-        self.assertAlmostEqual(df_elem["maximum block"][0], 3, 1)
+        self.assertAlmostEqual(df_elem["PymatgenData minimum melting_point"][0], 54.8, 1)
+        self.assertTrue(math.isnan(df_elem["PymatgenData maximum bulk_modulus"][0]))
+        self.assertAlmostEqual(df_elem["PymatgenData range X"][0], 1.61, 1)
+        self.assertAlmostEqual(df_elem["PymatgenData mean X"][0], 2.796, 1)
+        self.assertAlmostEqual(df_elem["PymatgenData maximum block"][0], 3, 1)
+
+    def test_elem_matscholar_el(self):
+        df_elem = ElementProperty.from_preset("matscholar_el").featurize_dataframe(self.df, col_id="composition")
+        self.assertAlmostEqual(df_elem["MatscholarElementData range embedding 149"].iloc[0],
+                               0.06827970966696739)
+        self.assertAlmostEqual(df_elem["MatscholarElementData range embedding 149"].iloc[1],
+                               0.06827970966696739)
+        self.assertAlmostEqual(df_elem["MatscholarElementData mean embedding 18"].iloc[0],
+                               -0.020534400502219795)
+        self.assertAlmostEqual(df_elem["MatscholarElementData mean embedding 18"].iloc[1],
+                               -0.02483355056028813)
+
+    def test_elem_megnet_el(self):
+        ep = ElementProperty.from_preset("megnet_el")
+        df_elem = ep.featurize_dataframe(self.df, col_id="composition")
+        self.assertAlmostEqual(df_elem["MEGNetElementData maximum embedding 1"].iloc[0], 0.127333, places=6)
+        self.assertAlmostEqual(df_elem["MEGNetElementData maximum embedding 1"].iloc[1], 0.127333, places=6)
+        self.assertAlmostEqual(df_elem["MEGNetElementData maximum embedding 11"].iloc[0], 0.160505, places=6)
+        self.assertAlmostEqual(df_elem["MEGNetElementData maximum embedding 11"].iloc[1], 0.160505, places=6)
+        self.assertTrue(ep.citations())
+
+    def test_meredig(self):
+        df_val = Meredig().featurize_dataframe(self.df, col_id="composition")
+        self.assertAlmostEqual(df_val["Fe fraction"].iloc[0], 2.0/5.0)
+        self.assertAlmostEqual(df_val["Fe fraction"].iloc[1], 0.5)
+        self.assertAlmostEqual(df_val["O fraction"].iloc[0], 3.0/5.0)
+        self.assertAlmostEqual(df_val["O fraction"].iloc[1], 0.5)
+        self.assertAlmostEqual(df_val["frac s valence electrons"].iloc[0], 0.294117647)
+        self.assertAlmostEqual(df_val["mean Number"].iloc[0], 15.2)
 
     def test_valence(self):
         df_val = ValenceOrbital().featurize_dataframe(self.df, col_id="composition")
@@ -90,12 +127,12 @@ class CompositionFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(df_ionic["avg ionic char"][0], 0.114461319)
 
         # Test 'fast'
-        self.assertEquals(1.0, featurizer.featurize(Composition("Fe3O4"))[0])
+        self.assertEqual(1.0, featurizer.featurize(Composition("Fe3O4"))[0])
         featurizer.fast = True
-        self.assertEquals(0, featurizer.featurize(Composition("Fe3O4"))[0])
+        self.assertEqual(0, featurizer.featurize(Composition("Fe3O4"))[0])
 
         # Make sure 'fast' works if I use-precomputed oxidation states
-        self.assertEquals(1, featurizer.featurize(Composition({
+        self.assertEqual(1, featurizer.featurize(Composition({
             Specie('Fe', 2): 1,
             Specie('Fe', 3): 2,
             Specie('O', -2): 4
@@ -128,11 +165,11 @@ class CompositionFeaturesTest(PymatgenTest):
                                        stats=["minimum", "maximum", "range", "mean", "std_dev"],
                                        data_source="deml")\
             .featurize_dataframe(self.df, col_id="composition")
-        self.assertAlmostEqual(df_fere_corr["minimum FERE correction"][0], -0.15213431610903)
-        self.assertAlmostEqual(df_fere_corr["maximum FERE correction"][0], 0.23)
-        self.assertAlmostEqual(df_fere_corr["range FERE correction"][0], 0.382134316)
-        self.assertAlmostEqual(df_fere_corr["mean FERE correction"][0], 0.077146274)
-        self.assertAlmostEqual(df_fere_corr["std_dev FERE correction"][0], 0.270209766)
+        self.assertAlmostEqual(df_fere_corr["DemlData minimum FERE correction"][0], -0.15213431610903)
+        self.assertAlmostEqual(df_fere_corr["DemlData maximum FERE correction"][0], 0.23)
+        self.assertAlmostEqual(df_fere_corr["DemlData range FERE correction"][0], 0.382134316)
+        self.assertAlmostEqual(df_fere_corr["DemlData mean FERE correction"][0], 0.077146274)
+        self.assertAlmostEqual(df_fere_corr["DemlData std_dev FERE correction"][0], 0.270209766)
 
     def test_atomic_orbitals(self):
         df_atomic_orbitals = AtomicOrbitals().featurize_dataframe(self.df, col_id="composition")
@@ -162,8 +199,8 @@ class CompositionFeaturesTest(PymatgenTest):
     def test_oxidation_states(self):
         featurizer = OxidationStates.from_preset("deml")
         features = dict(zip(featurizer.feature_labels(), featurizer.featurize(self.df["composition"][1])))
-        self.assertAlmostEquals(4, features["range oxidation state"])
-        self.assertAlmostEquals(2, features["maximum oxidation state"])
+        self.assertAlmostEqual(4, features["range oxidation state"])
+        self.assertAlmostEqual(2, features["maximum oxidation state"])
 
     def test_cohesive_energy(self):
         mpr = MPRester()
@@ -172,11 +209,29 @@ class CompositionFeaturesTest(PymatgenTest):
         df_cohesive_energy = CohesiveEnergy().featurize_dataframe(self.df, col_id="composition")
         self.assertAlmostEqual(df_cohesive_energy["cohesive energy"][0], 5.15768, 2)
 
+    def test_cohesive_energy_mp(self):
+        mpr = MPRester()
+        if not mpr.api_key:
+            raise SkipTest("Materials Project API key not set; Skipping cohesive energy test")
+        df_cohesive_energy = CohesiveEnergyMP().featurize_dataframe(self.df, col_id="composition")
+        self.assertAlmostEqual(df_cohesive_energy["cohesive energy (MP)"][0], 5.945, 2)
+
     def test_miedema_all(self):
         df = pd.DataFrame({"composition": [Composition("TiZr"),
                                            Composition("Mg10Cu50Ca40"),
                                            Composition("Fe2O3")]})
         miedema = Miedema(struct_types='all')
+        self.assertTrue(miedema.precheck(df["composition"].iloc[0]))
+        self.assertFalse(miedema.precheck(df["composition"].iloc[-1]))
+        self.assertAlmostEqual(miedema.precheck_dataframe(df, "composition"), 2 / 3)
+
+        # test precheck for oxidation-state decorated compositions
+        df = CompositionToOxidComposition(return_original_on_error=True).\
+            featurize_dataframe(df, 'composition')
+        self.assertTrue(miedema.precheck(df["composition_oxid"].iloc[0]))
+        self.assertFalse(miedema.precheck(df["composition_oxid"].iloc[-1]))
+        self.assertAlmostEqual(miedema.precheck_dataframe(df, "composition_oxid"), 2 / 3)
+
         mfps = miedema.featurize_dataframe(df, col_id="composition")
         self.assertAlmostEqual(mfps['Miedema_deltaH_inter'][0], -0.003445022152)
         self.assertAlmostEqual(mfps['Miedema_deltaH_amor'][0], 0.0707658836300)
@@ -190,6 +245,13 @@ class CompositionFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(math.isnan(mfps['Miedema_deltaH_amor'][2]), True)
         self.assertAlmostEqual(math.isnan(mfps['Miedema_deltaH_ss_min'][2]), True)
 
+        # make sure featurization works equally for compositions with or without
+        # oxidation states
+        mfps = miedema.featurize_dataframe(df, col_id="composition_oxid")
+        self.assertAlmostEqual(mfps['Miedema_deltaH_inter'][0], -0.003445022152)
+        self.assertAlmostEqual(mfps['Miedema_deltaH_amor'][0], 0.0707658836300)
+        self.assertAlmostEqual(mfps['Miedema_deltaH_ss_min'][0], 0.03663599755)
+
     def test_miedema_ss(self):
         df = pd.DataFrame({"composition": [Composition("TiZr"),
                                            Composition("Mg10Cu50Ca40"),
@@ -216,11 +278,23 @@ class CompositionFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(math.isnan(mfps['Miedema_deltaH_ss_no_latt'][2]), True)
 
     def test_yang(self):
-        comps = list(map(Composition, ["ZrHfTiCuNi", "CuNi",
-                                       "CoCrFeNiCuAl0.3", "CoCrFeNiCuAl"]))
+        comps = list(map(Composition,
+                         ["ZrHfTiCuNi", "CuNi", "CoCrFeNiCuAl0.3",
+                          "CoCrFeNiCuAl", "LaO3"]))
 
         # Run the featurization
         feat = YangSolidSolution()
+
+        df = pd.DataFrame({"composition": comps})
+        self.assertFalse(feat.precheck(df["composition"].iloc[-1]))
+        self.assertAlmostEqual(feat.precheck_dataframe(df, "composition"), 0.8, places=2)
+
+        # test precheck for oxidation-state decorated compositions
+        df = CompositionToOxidComposition(return_original_on_error=True). \
+            featurize_dataframe(df, 'composition')
+        self.assertFalse(feat.precheck(df["composition_oxid"].iloc[-1]))
+        self.assertAlmostEqual(feat.precheck_dataframe(df, "composition_oxid"), 0.8, places=2)
+
         feat.set_n_jobs(1)
         features = feat.featurize_many(comps)
 
@@ -231,7 +305,7 @@ class CompositionFeaturesTest(PymatgenTest):
         # I use a high tolerance because matminer uses a different source
         #   of radii than the original paper (do not have Kittel's atomic
         #   radii available)
-        self.assertEqual((4, 2), np.array(features).shape)
+        self.assertEqual((5, 2), np.array(features).shape)
         self.assertArrayAlmostEqual([0.95, 0.1021], features[0], decimal=2)
         self.assertArrayAlmostEqual([2.22, 0.0], features[1], decimal=2)
         self.assertArrayAlmostEqual([158.5, 0.0315], features[2], decimal=1)
@@ -263,7 +337,7 @@ class CompositionFeaturesTest(PymatgenTest):
             ef.featurize_many(stable_clusters),
             n_neighbors=1)
         self.assertArrayAlmostEqual([[0]]*8, ds)
-        self.assertEquals(8, nn_lookup._fit_X.shape[0])
+        self.assertEqual(8, nn_lookup._fit_X.shape[0])
 
         # Swap the order of the clusters, make sure it gets the same list
         nn_lookup_swapped = f.create_cluster_lookup_tool([Element('Zr'),
@@ -272,22 +346,22 @@ class CompositionFeaturesTest(PymatgenTest):
                                     sorted(nn_lookup_swapped._fit_X.tolist()))
 
         # Make sure we had a cache hit
-        self.assertEquals(1, f._create_cluster_lookup_tool.cache_info().misses)
-        self.assertEquals(1, f._create_cluster_lookup_tool.cache_info().hits)
+        self.assertEqual(1, f._create_cluster_lookup_tool.cache_info().misses)
+        self.assertEqual(1, f._create_cluster_lookup_tool.cache_info().hits)
 
         # Change the tolerance, see if it changes the results properly
         f.threshold = 0.002
         nn_lookup = f.create_cluster_lookup_tool([Element('Cu'),
                                                   Element('Zr')])
-        self.assertEquals(2, nn_lookup._fit_X.shape[0])
+        self.assertEqual(2, nn_lookup._fit_X.shape[0])
         ds, _ = nn_lookup.kneighbors(
             ef.featurize_many([Composition('CuZr10'), Composition('Cu3Zr12')]),
             n_neighbors=1)
         self.assertArrayAlmostEqual([[0]]*2, ds)
 
         # Make sure we had a cache miss
-        self.assertEquals(2, f._create_cluster_lookup_tool.cache_info().misses)
-        self.assertEquals(1, f._create_cluster_lookup_tool.cache_info().hits)
+        self.assertEqual(2, f._create_cluster_lookup_tool.cache_info().misses)
+        self.assertEqual(1, f._create_cluster_lookup_tool.cache_info().hits)
 
         # Compute the distances from Cu50Zr50
         mean_dists = f.compute_nearest_cluster_distance(Composition('CuZr'))
@@ -302,9 +376,9 @@ class CompositionFeaturesTest(PymatgenTest):
 
         # Test the dataframe calculator
         df = pd.DataFrame({'comp': [Composition('CuZr')]})
-        f.featurize_dataframe(df, 'comp')
+        df = f.featurize_dataframe(df, 'comp')
 
-        self.assertEquals(6, len(df.columns))
+        self.assertEqual(6, len(df.columns))
         self.assertIn('dist from 5 clusters |APE| < 0.002', df.columns)
 
         self.assertAlmostEqual(0.003508794,
