@@ -3,11 +3,14 @@
 # Distributed under the terms of the MIT License.
 
 from __future__ import unicode_literals, division
-
+import os
+import copy
 import unittest
-
+import csv
+import json
 import numpy as np
 import pandas as pd
+from multiprocessing import set_start_method
 from sklearn.exceptions import NotFittedError
 
 from pymatgen import Structure, Lattice, Molecule
@@ -22,7 +25,18 @@ from matminer.featurizers.structure import DensityFeatures, \
     SineCoulombMatrix, OrbitalFieldMatrix, GlobalSymmetryFeatures, \
     EwaldEnergy, BondFractions, BagofBonds, StructuralHeterogeneity, \
     MaximumPackingEfficiency, ChemicalOrdering, StructureComposition, \
-    Dimensionality, XRDPowderPattern, JarvisCFID
+    Dimensionality, XRDPowderPattern, CGCNNFeaturizer, JarvisCFID, \
+    SOAP, GlobalInstabilityIndex, \
+    StructuralComplexity
+
+# For the CGCNNFeaturizer
+try:
+    import torch
+    import cgcnn
+except ImportError:
+    torch, cgcnn = None, None
+
+test_dir = os.path.join(os.path.dirname(__file__))
 
 
 class StructureFeaturesTest(PymatgenTest):
@@ -76,18 +90,25 @@ class StructureFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(f[1], 23.046, 2)
         self.assertAlmostEqual(f[2], 0.620, 2)
 
+        nacl_disordered = copy.deepcopy(self.nacl)
+        nacl_disordered.replace_species({"Cl1-": "Cl0.99H0.01"})
+        self.assertFalse(df.precheck(nacl_disordered))
+        structures = [self.diamond, self.nacl, nacl_disordered]
+        df2 = pd.DataFrame({"structure": structures})
+        self.assertAlmostEqual(df.precheck_dataframe(df2, "structure"), 2 / 3)
+
     def test_global_symmetry(self):
         gsf = GlobalSymmetryFeatures()
         self.assertEqual(gsf.featurize(self.diamond), [227, "cubic", 1, True])
 
     def test_dimensionality(self):
         cscl = PymatgenTest.get_structure("CsCl")
+        graphite = PymatgenTest.get_structure("Graphite")
 
-        df = Dimensionality(bonds={("Cs", "Cl"): 3.5})
-        self.assertEqual(df.featurize(cscl)[0], 1)
+        df = Dimensionality()
 
-        df = Dimensionality(bonds={("Cs", "Cl"): 3.7})
         self.assertEqual(df.featurize(cscl)[0], 3)
+        self.assertEqual(df.featurize(graphite)[0], 2)
 
     def test_rdf_and_peaks(self):
         ## Test diamond
@@ -242,35 +263,71 @@ class StructureFeaturesTest(PymatgenTest):
             d["distances"]) - 1]), 7275)
 
     def test_coulomb_matrix(self):
+        # flat
+        cm = CoulombMatrix(flatten=True)
+        df = pd.DataFrame({"s": [self.diamond, self.nacl]})
+        with self.assertRaises(NotFittedError):
+            df = cm.featurize_dataframe(df, "s")
+        df = cm.fit_featurize_dataframe(df, "s")
+        labels = cm.feature_labels()
+        self.assertListEqual(labels,
+                             ["coulomb matrix eig 0", "coulomb matrix eig 1"])
+        self.assertArrayAlmostEqual(df[labels].iloc[0],
+                                    [49.169453, 24.546758],
+                                    decimal=5)
+        self.assertArrayAlmostEqual(df[labels].iloc[1],
+                                    [153.774731, 452.894322],
+                                    decimal=5)
+
+        # matrix
         species = ["C", "C", "H", "H"]
         coords = [[0, 0, 0], [0, 0, 1.203], [0, 0, -1.06], [0, 0, 2.263]]
         acetylene = Molecule(species, coords)
-        morig = CoulombMatrix().featurize(acetylene)
+        morig = CoulombMatrix(flatten=False).featurize(acetylene)
         mtarget = [[36.858, 15.835391290, 2.995098235, 1.402827813], \
                    [15.835391290, 36.858, 1.4028278132103624, 2.9950982], \
                    [2.9368896127, 1.402827813, 0.5, 0.159279959], \
                    [1.4028278132, 2.995098235, 0.159279959, 0.5]]
         self.assertAlmostEqual(
             int(np.linalg.norm(morig - np.array(mtarget))), 0)
-        m = CoulombMatrix(False).featurize(acetylene)[0]
+        m = CoulombMatrix(diag_elems=False,
+                          flatten=False).featurize(acetylene)[0]
         self.assertAlmostEqual(m[0][0], 0.0)
         self.assertAlmostEqual(m[1][1], 0.0)
         self.assertAlmostEqual(m[2][2], 0.0)
         self.assertAlmostEqual(m[3][3], 0.0)
 
     def test_sine_coulomb_matrix(self):
-        scm = SineCoulombMatrix()
+        # flat
+        scm = SineCoulombMatrix(flatten=True)
+        df = pd.DataFrame({"s": [self.sc, self.ni3al]})
+        with self.assertRaises(NotFittedError):
+            df = scm.featurize_dataframe(df, "s")
+        df = scm.fit_featurize_dataframe(df, "s")
+        labels = scm.feature_labels()
+        self.assertEqual(labels[0], "sine coulomb matrix eig 0")
+        self.assertArrayAlmostEqual(
+            df[labels].iloc[0],
+            [235.740418, 0.0, 0.0, 0.0],
+            decimal=5)
+        self.assertArrayAlmostEqual(
+            df[labels].iloc[1],
+            [232.578562, 1656.288171, 1403.106576, 1403.106576],
+            decimal=5)
+
+        # matrix
+        scm = SineCoulombMatrix(flatten=False)
         sin_mat = scm.featurize(self.diamond)
         mtarget = [[36.8581, 6.147068], [6.147068, 36.8581]]
         self.assertAlmostEqual(
             np.linalg.norm(sin_mat - np.array(mtarget)), 0.0, places=4)
-        scm = SineCoulombMatrix(False)
+        scm = SineCoulombMatrix(diag_elems=False, flatten=False)
         sin_mat = scm.featurize(self.diamond)[0]
         self.assertEqual(sin_mat[0][0], 0)
         self.assertEqual(sin_mat[1][1], 0)
 
     def test_orbital_field_matrix(self):
-        ofm_maker = OrbitalFieldMatrix()
+        ofm_maker = OrbitalFieldMatrix(flatten=False)
         ofm = ofm_maker.featurize(self.diamond)[0]
         mtarget = np.zeros((32, 32))
         mtarget[1][1] = 1.4789015  # 1.3675444
@@ -285,7 +342,7 @@ class StructureFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(
             np.linalg.norm(ofm - mtarget), 0.0, places=4)
 
-        ofm_maker = OrbitalFieldMatrix(True)
+        ofm_maker = OrbitalFieldMatrix(True, flatten=False)
         ofm = ofm_maker.featurize(self.diamond)[0]
         mtarget = np.zeros((39, 39))
         mtarget[1][1] = 1.4789015
@@ -350,7 +407,7 @@ class StructureFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(opvals[-1], 0.0, places=7)
 
         # Test coordination number
-        cn_fp = SiteStatsFingerprint.from_preset("JMolNN", stats=("mean",))
+        cn_fp = SiteStatsFingerprint.from_preset("JmolNN", stats=("mean",))
         cn_vals = cn_fp.featurize(self.diamond)
         self.assertEqual(cn_vals[0], 4.0)
 
@@ -415,25 +472,26 @@ class StructureFeaturesTest(PymatgenTest):
         df = bf_voronoi.featurize_dataframe(df, 's')
 
         # Ensure all data is properly labelled and organized
-        self.assertArrayEqual(df['C - C bond frac.'].as_matrix(), [1.0, np.nan])
-        self.assertArrayEqual(df['Al - Ni bond frac.'].as_matrix(), [np.nan, 0.5])
-        self.assertArrayEqual(df['Al - Al bond frac.'].as_matrix(), [np.nan, 0.0])
-        self.assertArrayEqual(df['Ni - Ni bond frac.'].as_matrix(), [np.nan, 0.5])
+        self.assertArrayEqual(df['C - C bond frac.'].to_numpy(), [1.0, np.nan])
+        self.assertArrayEqual(df['Al - Ni bond frac.'].to_numpy(), [np.nan, 0.5])
+        self.assertArrayEqual(df['Al - Al bond frac.'].to_numpy(), [np.nan, 0.0])
+        self.assertArrayEqual(df['Ni - Ni bond frac.'].to_numpy(), [np.nan, 0.5])
 
         # Test to make sure bad_bond_values (bbv) are still changed correctly
         # and check inplace behavior of featurize dataframe.
         bf_voronoi.bbv = 0.0
         df = pd.DataFrame.from_dict({'s': s_list})
         df = bf_voronoi.featurize_dataframe(df, 's')
-        self.assertArrayEqual(df['C - C bond frac.'].as_matrix(), [1.0, 0.0])
-        self.assertArrayEqual(df['Al - Ni bond frac.'].as_matrix(), [0.0, 0.5])
-        self.assertArrayEqual(df['Al - Al bond frac.'].as_matrix(), [0.0, 0.0])
-        self.assertArrayEqual(df['Ni - Ni bond frac.'].as_matrix(), [0.0, 0.5])
+        self.assertArrayEqual(df['C - C bond frac.'].to_numpy(), [1.0, 0.0])
+        self.assertArrayEqual(df['Al - Ni bond frac.'].to_numpy(), [0.0, 0.5])
+        self.assertArrayEqual(df['Al - Al bond frac.'].to_numpy(), [0.0, 0.0])
+        self.assertArrayEqual(df['Ni - Ni bond frac.'].to_numpy(), [0.0, 0.5])
 
     def test_bob(self):
 
         # Test a single fit and featurization
-        bob = BagofBonds(coulomb_matrix=SineCoulombMatrix(), token=' - ')
+        scm = SineCoulombMatrix(flatten=False)
+        bob = BagofBonds(coulomb_matrix=scm, token=' - ')
         bob.fit([self.ni3al])
         truth1 = [235.74041833262768, 1486.4464890775491, 1486.4464890775491,
                   1486.4464890775491, 38.69353092306119, 38.69353092306119,
@@ -448,22 +506,22 @@ class StructureFeaturesTest(PymatgenTest):
                          'Ni - Ni bond #0', 'Ni - Ni bond #1',
                          'Ni - Ni bond #2', 'Ni - Ni bond #3',
                          'Ni - Ni bond #4', 'Ni - Ni bond #5']
-        self.assertAlmostEqual(bob.featurize(self.ni3al), truth1)
+        self.assertArrayAlmostEqual(bob.featurize(self.ni3al), truth1)
         self.assertEqual(bob.feature_labels(), truth1_labels)
 
         # Test padding from fitting and dataframe featurization
-        bob.coulomb_matrix = CoulombMatrix()
+        bob.coulomb_matrix = CoulombMatrix(flatten=False)
         bob.fit([self.ni3al, self.cscl, self.diamond_no_oxi])
         df = pd.DataFrame({'structures': [self.cscl]})
         df = bob.featurize_dataframe(df, 'structures')
         self.assertEqual(len(df.columns.values), 25)
         self.assertAlmostEqual(df['Cs site #0'][0], 7513.468312122532)
         self.assertAlmostEqual(df['Al site #0'][0], 0.0)
-        self.assertAlmostEqual(df['Cs - Cl bond #1'][0], 135.74726437398044)
+        self.assertAlmostEqual(df['Cs - Cl bond #1'][0], 135.74726437398044, 3)
         self.assertAlmostEqual(df['Al - Ni bond #0'][0], 0.0)
 
         # Test error handling for bad fits or null fits
-        bob = BagofBonds()
+        bob = BagofBonds(CoulombMatrix(flatten=False))
         self.assertRaises(NotFittedError, bob.featurize, self.nacl)
         bob.fit([self.ni3al, self.diamond])
         self.assertRaises(ValueError, bob.featurize, self.nacl)\
@@ -592,6 +650,182 @@ class StructureFeaturesTest(PymatgenTest):
         self.assertEqual(len(pattern), 91)
         self.assertEqual(len(xpp.feature_labels()), 91)
 
+    @unittest.skipIf(not (torch and cgcnn),
+                     "pytorch or cgcnn not installed.")
+    def test_cgcnn_featurizer(self):
+        # Test regular classification.
+        cla_props, cla_atom_features, cla_structs = self._get_cgcnn_data()
+        atom_fea_len = 64
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(atom_init_fea=cla_atom_features,
+                            train_size=5, val_size=2, test_size=3,
+                            atom_fea_len=atom_fea_len)
+
+        cgcnn_featurizer.fit(X=cla_structs, y=cla_props)
+        self.assertEqual(len(cgcnn_featurizer.feature_labels()), atom_fea_len)
+        state_dict = cgcnn_featurizer.model.state_dict()
+        self.assertEqual(state_dict['embedding.weight'].size(),
+                         torch.Size([64, 92]))
+        self.assertEqual(state_dict['embedding.bias'].size(),
+                         torch.Size([64]))
+        self.assertEqual(state_dict['convs.0.fc_full.weight'].size(),
+                         torch.Size([128, 169]))
+        self.assertEqual(state_dict['convs.1.bn1.weight'].size(),
+                         torch.Size([128]))
+        self.assertEqual(state_dict['convs.2.bn2.bias'].size(),
+                         torch.Size([64]))
+        self.assertEqual(state_dict['conv_to_fc.weight'].size(),
+                         torch.Size([128, 64]))
+        self.assertEqual(state_dict['fc_out.weight'].size(),
+                         torch.Size([2, 128]))
+
+        for struct in cla_structs:
+            result = cgcnn_featurizer.featurize(struct)
+            self.assertEqual(len(result), atom_fea_len)
+
+        # Test regular regression and default atom_init_fea and featurize_many.
+        reg_props, reg_atom_features, reg_structs = \
+            self._get_cgcnn_data("regression")
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(task="regression", atom_fea_len=atom_fea_len,
+                            train_size=6, val_size=2, test_size=2)
+
+        cgcnn_featurizer.fit(X=reg_structs, y=reg_props)
+        cgcnn_featurizer.set_n_jobs(1)
+
+        result = cgcnn_featurizer.featurize_many(entries=reg_structs)
+        self.assertEqual(np.array(result).shape,
+                         (len(reg_structs), atom_fea_len))
+
+        # Test classification from pre-trained model.
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(h_fea_len=32, n_conv=4,
+                            pretrained_name='semi-metal-classification',
+                            atom_init_fea=cla_atom_features, train_size=5,
+                            val_size=2, test_size=3, atom_fea_len=atom_fea_len)
+        cgcnn_featurizer.fit(X=cla_structs, y=cla_props)
+        self.assertEqual(len(cgcnn_featurizer.feature_labels()), atom_fea_len)
+
+        validate_features = [2.1295, 2.1288, 1.8504, 1.9175, 2.1094,
+                             1.7770, 2.0471, 1.7426, 1.7288, 1.7770]
+        for struct, validate_feature in zip(cla_structs, validate_features):
+            result = cgcnn_featurizer.featurize(struct)
+            self.assertEqual(len(result), atom_fea_len)
+            self.assertAlmostEqual(result[0], validate_feature, 4)
+
+        # Test regression from pre-trained model.
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(task="regression", h_fea_len=32, n_conv=4,
+                            pretrained_name='formation-energy-per-atom',
+                            atom_init_fea=reg_atom_features,
+                            train_size=5, val_size=2, test_size=3,
+                            atom_fea_len=atom_fea_len)
+        cgcnn_featurizer.fit(X=reg_structs, y=reg_props)
+        self.assertEqual(len(cgcnn_featurizer.feature_labels()), atom_fea_len)
+
+        validate_features = [1.6871, 1.5679, 1.5316, 1.6419, 1.6031,
+                             1.4333, 1.5709, 1.5070, 1.5038, 1.4333]
+
+        for struct, validate_feature in zip(reg_structs, validate_features):
+            result = cgcnn_featurizer.featurize(struct)
+            self.assertEqual(len(result), atom_fea_len)
+            self.assertAlmostEqual(result[-1], validate_feature, 4)
+
+        # Test warm start regression.
+        warm_start_file = os.path.join(test_dir,
+                                       'cgcnn_test_regression_model.pth.tar')
+        warm_start_model = torch.load(warm_start_file)
+        self.assertEqual(warm_start_model['epoch'], 31)
+        self.assertEqual(warm_start_model['best_epoch'], 9)
+        self.assertAlmostEqual(warm_start_model['best_mae_error'].numpy(),
+                               2.3700, 4)
+
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(task="regression", warm_start_file=warm_start_file,
+                            epochs=100, atom_fea_len=atom_fea_len,
+                            atom_init_fea=reg_atom_features,
+                            train_size=6, val_size=2, test_size=2)
+        cgcnn_featurizer.fit(X=reg_structs, y=reg_props)
+
+        # If use CGCNN featurize_many(), you should change the multiprocessing
+        # start_method to 'spawn', because Gloo (that uses Infiniband) and
+        # NCCL2 are not fork safe, pytorch don't support them or just
+        # set n_jobs = 1 to avoid multiprocessing as follows.
+        set_start_method('spawn', force=True)
+        result = cgcnn_featurizer.featurize_many(entries=reg_structs)
+        self.assertEqual(np.array(result).shape,
+                         (len(reg_structs), atom_fea_len))
+
+        # Test featurize_dataframe.
+        df = pd.DataFrame.from_dict({"structure": cla_structs})
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(atom_init_fea=cla_atom_features,
+                            train_size=5, val_size=2, test_size=3,
+                            atom_fea_len=atom_fea_len)
+        cgcnn_featurizer.fit(X=df["structure"], y=cla_props)
+        self.assertEqual(len(cgcnn_featurizer.feature_labels()), atom_fea_len)
+        cgcnn_featurizer.set_n_jobs(1)
+        result = cgcnn_featurizer.featurize_dataframe(df, "structure")
+        self.assertTrue("CGCNN_feature_{}".format(atom_fea_len - 1)
+                        in result.columns)
+        self.assertEqual(np.array(result).shape,
+                         (len(reg_structs), atom_fea_len + 1))
+
+        # Test fit_featurize_dataframe.
+        df = pd.DataFrame.from_dict({"structure": cla_structs})
+        cgcnn_featurizer = \
+            CGCNNFeaturizer(atom_init_fea=cla_atom_features,
+                            train_size=5, val_size=2, test_size=3,
+                            atom_fea_len=atom_fea_len)
+        result = cgcnn_featurizer.fit_featurize_dataframe(
+            df, "structure", fit_args=[cla_props])
+        self.assertEqual(len(cgcnn_featurizer.feature_labels()), atom_fea_len)
+        self.assertTrue("CGCNN_feature_{}".format(atom_fea_len - 1)
+                        in result.columns)
+        self.assertEqual(np.array(result).shape,
+                         (len(reg_structs), atom_fea_len + 1))
+
+    @staticmethod
+    def _get_cgcnn_data(task="classification"):
+        """
+        Get cgcnn sample data.
+        Args:
+            task (str): Classification or regression,
+                        decided which sample data to return.
+
+        Returns:
+            id_prop_data (list): List of property data.
+            elem_embedding (list): List of element features.
+            struct_list (list): List of structure object.
+        """
+        if task == "classification":
+            cgcnn_data_path = os.path.join(os.path.dirname(cgcnn.__file__),
+                                           "..",
+                                           "data", "sample-classification")
+        else:
+            cgcnn_data_path = os.path.join(os.path.dirname(cgcnn.__file__),
+                                           "..",
+                                           "data", "sample-regression")
+
+        struct_list = list()
+        cif_list = list()
+        with open(os.path.join(cgcnn_data_path, "id_prop.csv")) as f:
+            reader = csv.reader(f)
+            id_prop_data = [row[1] for row in reader]
+        with open(os.path.join(cgcnn_data_path, "atom_init.json")) as f:
+            elem_embedding = json.load(f)
+
+        for file in os.listdir(cgcnn_data_path):
+            if file.endswith('.cif'):
+                cif_list.append(int(file[:-4]))
+                cif_list = sorted(cif_list)
+        for cif_name in cif_list:
+            crystal = Structure.from_file(os.path.join(cgcnn_data_path,
+                                                       '{}.cif'.format(
+                                                           cif_name)))
+            struct_list.append(crystal)
+        return id_prop_data, elem_embedding, struct_list
+
     def test_jarvisCFID(self):
 
         # default (all descriptors)
@@ -610,6 +844,84 @@ class StructureFeaturesTest(PymatgenTest):
         self.assertAlmostEqual(fvec[-1], 24, places=3)
         self.assertAlmostEqual(fvec[0], 0, places=3)
 
+    def test_SOAP(self):
+        # Test individual samples
+        soap = SOAP(n_max=4, l_max=2, r_cut=3.0)
+        soap.fit([self.diamond])
+        v = soap.featurize(self.diamond)
+        self.assertEqual(len(v), 30)
+        self.assertAlmostEqual(v[0], 5.299793243408203, places=6)
 
-if __name__ == '__main__':
-    unittest.main()
+        soap.fit([self.ni3al])
+        v = soap.featurize(self.ni3al)
+        self.assertEqual(len(v), 90)
+        self.assertAlmostEqual(v[0], 0.10329483449459076, places=6)
+
+        # Test dataframe fitting
+        df = pd.DataFrame({"s": [self.diamond, self.ni3al, self.nacl]})
+        soap.fit(df["s"])
+        df = soap.featurize_dataframe(df, "s", inplace=False)
+        self.assertTupleEqual(df.shape, (3, 451))
+        self.assertAlmostEqual(df["SOAP_449"].iloc[1], 3.029167413711548, places=5)
+
+    def test_GlobalInstabilityIndex(self):
+        # Test diamond and ni3al fail precheck
+        gii = GlobalInstabilityIndex(r_cut=4.0, disordered_pymatgen=False)
+        self.assertFalse(gii.precheck(self.diamond))
+        self.assertFalse(gii.precheck(self.ni3al))
+        # Test they raise errors when featurizing
+        with self.assertRaises(AttributeError):
+            gii.featurize(self.ni3al)
+        with self.assertRaises(ValueError):
+            gii.featurize(self.diamond)
+
+        # Ordinary case of nacl
+        self.assertTrue(gii.precheck(self.nacl))
+        self.assertAlmostEqual(gii.featurize(self.nacl)[0], 0.08491655709)
+
+        # Test bond valence sums are accurate for NaCl.
+        # Values are closer to 0.915 than 1.0 due to structure specified here.
+        # Using CollCode181148 from the ICSD, I see bond valence sums of 0.979
+        site1, site2 = (self.nacl[0], self.nacl[1])
+        neighs1 = self.nacl.get_neighbors(site1, 4)
+        neighs2 = self.nacl.get_neighbors(site2, 4)
+        site_val1 = site1.species.elements[0].oxi_state
+        site_el1 = str(site1.species.element_composition.elements[0])
+        site_val2 = site2.species.elements[0].oxi_state
+        site_el2 = str(site2.species.element_composition.elements[0])
+        self.assertAlmostEqual(gii.calc_bv_sum(site_val1,
+                                               site_el1,
+                                               neighs1), 0.9150834429025214)
+        self.assertAlmostEqual(gii.calc_bv_sum(site_val2,
+                                               site_el2,
+                                               neighs2), -0.915083442902522)
+
+        # Behavior when disorder is present
+        gii_pymat = GlobalInstabilityIndex(r_cut=4.0, disordered_pymatgen=True)
+        nacl_disordered = copy.deepcopy(self.nacl)
+        nacl_disordered.replace_species({"Cl1-": "Cl0.5Br0.5"})
+        nacl_disordered.add_oxidation_state_by_element({'Na': 1, 'Cl': -1, 'Br': -1})
+        self.assertTrue(gii.precheck(nacl_disordered))
+        with self.assertRaises(ValueError):
+            gii.featurize(nacl_disordered)
+        self.assertAlmostEqual(gii_pymat.featurize(nacl_disordered)[0], 0.39766464)
+
+    def test_structural_complexity(self):
+        s = Structure.from_file(
+            os.path.join(test_dir, "Dy2HfS5_mp-1198001_computed.cif")
+        )
+
+        featurizer = StructuralComplexity()
+        ig, igbits = featurizer.featurize(s)
+
+        self.assertAlmostEqual(2.5, ig, places=3)
+        self.assertAlmostEqual(80, igbits, places=3)
+
+        s = Structure.from_file(
+            os.path.join(test_dir, "Cs2CeN5O17_mp-1198000_computed.cif")
+        )
+
+        featurizer = StructuralComplexity()
+        ig, igbits = featurizer.featurize(s)
+
+        self.assertAlmostEqual(3.764, ig, places=3)
